@@ -54,14 +54,25 @@ setRefClass("H2OConnectionMutableState",
 #' is not found at port 54321.
 #' @slot ip A \code{character} string specifying the IP address of the H2O cloud.
 #' @slot port A \code{numeric} value specifying the port number of the H2O cloud.
+#' @slot https Set this to TRUE to use https instead of http.
+#' @slot insecure Set this to TRUE to disable SSL certificate checking.
+#' @slot username Username to login with.
+#' @slot password Password to login with.
 #' @slot mutable An \code{H2OConnectionMutableState} object to hold the mutable state for the H2O connection.
 #' @aliases H2OConnection
 #' @export
 setClass("H2OConnection",
-         representation(ip="character", port="numeric", mutable="H2OConnectionMutableState"),
-         prototype(ip      = NA_character_,
-                   port    = NA_integer_,
-                   mutable = new("H2OConnectionMutableState")))
+         representation(ip="character", port="numeric",
+                        https="logical", insecure="logical",
+                        username="character", password="character",
+                        mutable="H2OConnectionMutableState"),
+         prototype(ip       = NA_character_,
+                   port     = NA_integer_,
+                   https    = FALSE,
+                   insecure = FALSE,
+                   username = NA_character_,
+                   password = NA_character_,
+                   mutable  = new("H2OConnectionMutableState")))
 
 
 
@@ -353,6 +364,8 @@ setMethod("show", "H2OModel", function(object) {
   if( !is.null(model.parts$tm) ) print(model.parts$tm)
   cat("\n")
   if( !is.null(model.parts$vm) ) print(model.parts$vm)
+  cat("\n")
+  if( !is.null(model.parts$xm) ) print(model.parts$xm)
 })
 
 #'
@@ -387,7 +400,7 @@ setMethod("summary", "H2OModel", function(object, ...) {
   cat("\n")
 
   # VI could be real, true variable importances or GLM coefficients
-  haz_varimp <- !is.null(m$variable_importances) || !is.null(m$standardized_coefficients_magnitude)
+  haz_varimp <- !is.null(m$variable_importances) || !is.null(m$standardized_coefficient_magnitudes)
   if( haz_varimp ) {
     cat("Variable Importances: (Extract with `h2o.varimp`) \n")
     cat("=================================================\n\n")
@@ -397,24 +410,25 @@ setMethod("summary", "H2OModel", function(object, ...) {
 
 .showMultiMetrics <- function(o, which="Training") {
   arg <- "train"
-  if( which == "Validation" ) arg <- "valid"
+  if( which == "Validation" ) { arg <- "valid"
+  } else if ( which == "Cross-Validation" ) { arg <- "xval" }
   tm <- o@metrics
   cat(which, "Set Metrics: \n")
   cat("=====================\n")
   if( !is.null(tm$description)     )  cat(tm$description, "\n")
-  if( !is.null(tm$frame) && !is.null(tm$frame$name) )  cat("\nExtract", tolower(which),"frame with", paste0("`h2o.getFrame(\"",tm$frame$name, "\")`"))
-  if( !is.null(tm$MSE)                              )  cat("\nMSE: (Extract with `h2o.mse`)", tm$MSE)
-  if( !is.null(tm$r2)                               )  cat("\nR^2: (Extract with `h2o.r2`)", tm$r2)
-  if( !is.null(tm$logloss)                          )  cat("\nLogloss: (Extract with `h2o.logloss`)", tm$logloss)
-  if( !is.null(tm$AUC)                              )  cat("\nAUC: (Extract with `h2o.auc`)", tm$AUC)
-  if( !is.null(tm$Gini)                             )  cat("\nGini: (Extract with `h2o.gini`)", tm$Gini)
-  if( !is.null(tm$null_deviance)                    )  cat("\nNull Deviance: (Extract with `h2o.nulldeviance`)", tm$null_deviance)
-  if( !is.null(tm$residual_deviance)                )  cat("\nResidual Deviance: (Extract with `h2o.residual_deviance`)", tm$residual_deviance)
-  if( !is.null(tm$AIC)                              )  cat("\nAIC: (Extract with `h2o.aic`)", tm$AIC)
-  if( !is.null(tm$cm)                               )  cat(paste0("\nConfusion Matrix: Extract with `h2o.confusionMatrix(<model>,", arg, "=TRUE)`)\n"));
-  if( !is.null(tm$cm)                               )  { cat("=========================================================================\n"); print(data.frame(tm$cm$table)) }
-  if( !is.null(tm$hit_ratio_table)                  )  cat(paste0("\nHit Ratio Table: Extract with `h2o.hit_ratio_table(<model>,", arg, "=TRUE)`\n"))
-  if( !is.null(tm$hit_ratio_table)                  )  { cat("=======================================================================\n"); print(h2o.hit_ratio_table(tm$hit_ratio_table)); }
+  if( !is.null(tm[["frame"]]) && !is.null(tm[["frame"]][["name"]]) )  cat("\nExtract", tolower(which),"frame with", paste0("`h2o.getFrame(\"",tm$frame$name, "\")`"))
+  if( !is.null(tm$MSE)                                             )  cat("\nMSE: (Extract with `h2o.mse`)", tm$MSE)
+  if( !is.null(tm$r2)                                              )  cat("\nR^2: (Extract with `h2o.r2`)", tm$r2)
+  if( !is.null(tm$logloss)                                         )  cat("\nLogloss: (Extract with `h2o.logloss`)", tm$logloss)
+  if( !is.null(tm$AUC)                                             )  cat("\nAUC: (Extract with `h2o.auc`)", tm$AUC)
+  if( !is.null(tm$Gini)                                            )  cat("\nGini: (Extract with `h2o.gini`)", tm$Gini)
+  if( !is.null(tm$null_deviance)                                   )  cat("\nNull Deviance: (Extract with `h2o.nulldeviance`)", tm$null_deviance)
+  if( !is.null(tm$residual_deviance)                               )  cat("\nResidual Deviance: (Extract with `h2o.residual_deviance`)", tm$residual_deviance)
+  if( !is.null(tm$AIC)                                             )  cat("\nAIC: (Extract with `h2o.aic`)", tm$AIC)
+  if( !is.null(tm$cm)                                              )  { if ( arg != "xval" ) { cat(paste0("\nConfusion Matrix: Extract with `h2o.confusionMatrix(<model>,", arg, "=TRUE)`)\n")); } }
+  if( !is.null(tm$cm)                                              )  { if ( arg != "xval" ) { cat("=========================================================================\n"); print(data.frame(tm$cm$table)) } }
+  if( !is.null(tm$hit_ratio_table)                                 )  cat(paste0("\nHit Ratio Table: Extract with `h2o.hit_ratio_table(<model>,", arg, "=TRUE)`\n"))
+  if( !is.null(tm$hit_ratio_table)                                 )  { cat("=======================================================================\n"); print(h2o.hit_ratio_table(tm$hit_ratio_table)); }
   cat("\n")
   invisible(tm)
 }
@@ -539,8 +553,8 @@ setMethod("getClusterSizes", "H2OClusteringModel", function(object) { object@mod
 #' @aliases H2OModelMetrics
 #' @export
 setClass("H2OModelMetrics",
-         representation(algorithm="character", on_train="logical", metrics="listOrNull"),
-         prototype(algorithm=NA_character_, on_train=FALSE, metrics=NULL),
+         representation(algorithm="character", on_train="logical", on_valid="logical", on_xval="logical", metrics="listOrNull"),
+         prototype(algorithm=NA_character_, on_train=FALSE, on_valid=FALSE, on_xval=FALSE, metrics=NULL),
          contains="VIRTUAL")
 
 #' @rdname H2OModelMetrics-class
@@ -549,7 +563,8 @@ setClass("H2OModelMetrics",
 setMethod("show", "H2OModelMetrics", function(object) {
     cat(class(object), ": ", object@algorithm, "\n", sep="")
     if( object@on_train ) cat("** Reported on training data. **\n")
-    else                  cat("** Reported on validation data. **\n")
+    if( object@on_valid ) cat("** Reported on validation data. **\n")
+    if( object@on_xval ) cat("** Reported on cross-validation data. **\n")
     if( !is.null(object@metrics$description) ) cat("Description: ", object@metrics$description, "\n\n", sep="")
     else                                       cat("\n")
 })
@@ -595,7 +610,8 @@ setMethod("show", "H2OMultinomialMetrics", function(object) {
   if( !is.null(object@metrics) ) {
     callNextMethod(object)  # call super
     if( object@on_train ) .showMultiMetrics(object, "Training")
-    else                  .showMultiMetrics(object, "Validation")
+    if( object@on_valid ) .showMultiMetrics(object, "Validation")
+    if( object@on_xval ) .showMultiMetrics(object, "Cross-Validation")
   } else print(NULL)
 })
 #' @rdname H2OModelMetrics-class
@@ -607,6 +623,7 @@ setMethod("show", "H2ORegressionMetrics", function(object) {
   callNextMethod(object)
   cat("MSE:  ", object@metrics$MSE, "\n", sep="")
   cat("R2 :  ", h2o.r2(object), "\n", sep="")
+  cat("Mean Residual Deviance :  ", h2o.mean_residual_deviance(object), "\n", sep="")
   null_dev <- h2o.null_deviance(object)
   res_dev  <- h2o.residual_deviance(object)
   null_dof <- h2o.null_dof(object)
@@ -646,12 +663,25 @@ setMethod("show", "H2OAutoEncoderMetrics", function(object) {
     callNextMethod(object)  # call super
     object@metrics$frame$name <- NULL
     if( object@on_train ) .showMultiMetrics(object, "Training")
-    else                  .showMultiMetrics(object, "Validation")
+    if( object@on_valid ) .showMultiMetrics(object, "Validation")
+    if( object@on_xval ) .showMultiMetrics(object, "Cross-Validation")
   } else print(NULL)
 })
 #' @rdname H2OModelMetrics-class
 #' @export
 setClass("H2ODimReductionMetrics", contains="H2OModelMetrics")
+#' @rdname H2OModelMetrics-class
+#' @export
+setMethod("show", "H2ODimReductionMetrics", function(object) {
+  if( !is.null(object@metrics) ) {
+    callNextMethod(object)
+    m <- object@metrics
+    if( object@algorithm == "glrm" ) {
+      cat("Sum of Squared Error (Numeric): ", m$numerr)
+      cat("\nMisclassification Error (Categorical): ", m$caterr)
+    }
+  } else print(NULL)
+})
 
 #' H2O Future Model
 #'
@@ -699,4 +729,114 @@ str.H2OFrame <- function(object, cols=FALSE, ...) {
         cat("num ", paste(first.10.rows, collapse=' '), if( nr > 10L ) " ...", "\n", sep="")
     }
   }
-}
\ No newline at end of file
+}
+#' H2O Grid
+#'
+#' A class to contain the information about grid results
+#' @slot conn an \linkS4class{H2OConnection}
+#' @slot grid_id the final identifier of grid
+#' @slot model_ids  list of model IDs which are included in the grid object
+#' @slot hyper_names  list of parameter names used for grid search
+#' @slot failed_params  list of model parameters which caused a failure during model building, 
+#'                      it can contain a null value
+#' @slot failure_details  list of detailed messages which correspond to failed parameters field
+#' @slot failure_stack_traces  list of stack traces corresponding to model failures reported by
+#'                             failed_params and failure_details fields
+#' @slot failed_raw_params list of failed raw parameters
+#' @seealso \linkS4class{H2OModel} for the final model types.
+#' @aliases H2OGrid
+#' @export
+setClass("H2OGrid", representation(conn = "H2OConnection",
+                                   grid_id = "character",
+                                   model_ids = "list",
+                                   hyper_names = "list",
+                                   failed_params = "list",
+                                   failure_details = "list",
+                                   failure_stack_traces = "list",
+                                   failed_raw_params = "matrix"))
+
+#' Format grid object in user-friendly way
+#'
+#' @rdname H2OGrid-class
+#' @param object an \code{H2OGrid} object.
+#' @export
+setMethod("show", "H2OGrid", function(object) {
+  cat("H2O Grid Details\n")
+  cat("================\n\n")
+  cat("Grid ID:", object@grid_id, "\n")
+  cat("Used hyper parameters: \n")
+  lapply(object@hyper_names, function(name) { cat("  - ", name, "\n") })
+  cat("Number of models:", length(object@model_ids), "\n")
+  cat("Number of failed models:", length(object@failed_params), "\n\n")
+  hyper_names <- object@hyper_names
+  model_ids <- sapply(object@model_ids, function(model_id) { model_id })
+  if (length(object@model_ids) > 0) {
+    # Fetch all models
+    all_models <- lapply(object@model_ids, function(model_id) { h2o.getModel(model_id) })
+    # Extract hyper parameters from models
+    params_ok <- lapply(hyper_names, function(name) {
+                      sapply(all_models, function(model) {
+                             v <- model@allparameters[[name]]
+                             if ((is.list(v) || is.array(v) || is.vector(v)) && length(v) > 1) {
+                               .collapse(v)
+                             } else {
+                               v
+                             }
+                      })
+                    })
+    names(params_ok) <- hyper_names # Assign correct names to items in params list
+    status_ok <- rep("OK", length(object@model_ids))
+    cat("Generated models\n")
+    cat("----------------\n")
+    if ( length(params_ok) > 0 ) { print(data.frame(params_ok, status_ok, model_ids), row.names = FALSE)
+    } else {                       print(data.frame(           status_ok, model_ids), row.names = FALSE) }
+
+  }
+  if (length(object@failed_params) > 0) {
+    # Extract failed parameters info
+    params_failed <- lapply(1:length(hyper_names), function(idx) {
+                      apply(object@failed_raw_params, 1, function(fp) { fp[idx] })
+                    })
+    names(params_failed) <- hyper_names
+    status_failed <- rep("FAIL", length(object@failed_params))
+    msgs_failed <- sapply(object@failure_details, function(msg) { paste0("\"", msg, "\"") })
+    df_failed <- data.frame(params_failed, status_failed, msgs_failed)
+    cat("Failed models\n")
+    cat("-------------\n")
+    print(df_failed, row.names = FALSE)
+  }
+})
+#' Format grid object in user-friendly way
+#'
+#' @param object an \code{H2OGrid} object.
+#' @param show_stack_traces  a flag to show stack traces for model failures
+#' @export
+setMethod("summary", "H2OGrid",
+          function(object, show_stack_traces = F) {
+            show(object)
+            cat("H2O Grid Summary\n")
+            cat("================\n\n")
+            cat("Grid ID:", object@grid_id, "\n")
+            cat("Used hyper parameters: \n")
+            lapply(object@hyper_names, function(name) { cat("  - ", name, "\n") })
+            cat("Number of models:", length(object@model_ids), "\n")
+            if (length(object@model_ids) > 0) {
+              for (idx in 1:length(object@model_ids)) {
+                cat("  - ", object@model_ids[[idx]], "\n")
+              }
+            }
+            cat("\nNumber of failed models:", length(object@failed_params), "\n")
+            if (length(object@failed_params) > 0) {
+              for (idx in 1:length(object@failed_params)) {
+                cat("  - ", object@failure_details[[idx]])
+                if (show_stack_traces) {
+                  cat(object@failure_stack_traces[[idx]], "\n")
+                }
+              }
+            }
+
+            if (!show_stack_traces && length(object@failed_params) > 0) {
+              cat("\nNote: To see exception stack traces please pass parameter `show_stack_traces = T` to this function.\n")
+            }
+})
+
