@@ -8,18 +8,17 @@
 #' @param training_frame An \linkS4class{H2OFrame} object containing the variables in the model.
 #' @param model_id (Optional) The unique id assigned to the resulting model. If
 #'        none is given, an id will automatically be generated.
-#' @param overwrite_with_best_model Logcial. If \code{TRUE}, overwrite the final model with the best model found during training. Defaults to \code{TRUE}.
-#' @param n_folds (Optional) Number of folds for cross-validation. If \code{nfolds >= 2}, then \code{validation} must remain empty.
-#' @param validation_frame (Optional) An \code{\link{H2OFrame}} object indicating the validation dataset used to contruct the confusion matrix. If left blank, this defaults to the training data when \code{nfolds = 0}
+#' @param overwrite_with_best_model Logical. If \code{TRUE}, overwrite the final model with the best model found during training. Defaults to \code{TRUE}.
+#' @param validation_frame (Optional) An \code{\link{H2OFrame}} object indicating the validation dataset used to construct the confusion matrix. If left blank, this defaults to the training data when \code{nfolds = 0}
 #' @param checkpoint "Model checkpoint (either key or H2ODeepLearningModel) to resume training with."
 #' @param autoencoder Enable auto-encoder for model building.
 #' @param use_all_factor_levels \code{Logical}. Use all factor levels of categorical variance.
-#'        Otherwise the first factor level is omittted (without loss of accuracy). Useful for
-#'        variable imporotances and auto-enabled for autoencoder.
+#'        Otherwise the first factor level is omitted (without loss of accuracy). Useful for
+#'        variable importances and auto-enabled for autoencoder.
 #' @param activation A string indicating the activation function to use. Must be either "Tanh",
 #'        "TanhWithDropout", "Rectifier", "RectifierWithDropout", "Maxout", or "MaxoutWithDropout"
 #' @param hidden Hidden layer sizes (e.g. c(100,100))
-#' @param epochs How many times the dataset shoud be iterated (streamed), can be fractional
+#' @param epochs How many times the dataset should be iterated (streamed), can be fractional
 #' @param train_samples_per_iteration Number of training samples (globally) per MapReduce iteration.
 #'        Special values are: \bold{0} one epoch; \bold{-1} all available data (e.g., replicated
 #'        training data); or \bold{-2} auto-tuning (default)
@@ -32,24 +31,28 @@
 #' @param rate Learning rate (higher => less stable, lower => slower convergence)
 #' @param rate_annealing Learning rate annealing: \eqn{(rate)/(1 + rate_annealing*samples)}
 #' @param rate_decay Learning rate decay factor between layers (N-th layer: \eqn{rate*\alpha^(N-1)})
-#' @param momentum_start Initial momentum at the beginning of traning (try 0.5)
+#' @param momentum_start Initial momentum at the beginning of training (try 0.5)
 #' @param momentum_ramp Number of training samples for which momentum increases
-#' @param momentum_stable Final momentum after ther amp is over (try 0.99)
+#' @param momentum_stable Final momentum after the amp is over (try 0.99)
 #' @param nesterov_accelerated_gradient \code{Logical}. Use Nesterov accelerated gradient
 #'        (recommended)
 #' @param input_dropout_ratio A fraction of the features for each training row to be omitted from
 #'        training in order to improve generalization (dimension sampling).
-#' @param hidden_dropout_ratios Input layer dropout ration (can improve generalization) specify one
+#' @param hidden_dropout_ratios Input layer dropout ratio (can improve generalization) specify one
 #'        value per hidden layer, defaults to 0.5
-#' @param l1 L1 regularization (can add stability and improve generalization, cause many weights to
+#' @param l1 L1 regularization (can add stability and improve generalization, causes many weights to
 #'        become 0)
 #' @param l2 L2 regularization (can add stability and improve generalization, causes many weights to
 #'        be small)
 #' @param max_w2 Constraint for squared sum of incoming weights per unit (e.g. Rectifier)
 #' @param initial_weight_distribution Can be "Uniform", "UniformAdaptive", or "Normal"
-#' @param initial_weight_scale Unifrom: -value ... value, Normal: stddev
-#' @param loss Loss function: Automatic, CrossEntropy (for classification only), MeanSquare, Absolute
-#'        (experimental) or Huber (experimental)
+#' @param initial_weight_scale Uniform: -value ... value, Normal: stddev
+#' @param loss Loss function: "Automatic", "CrossEntropy" (for classification only), "MeanSquare", "Absolute"
+#'        (experimental) or "Huber" (experimental)
+#' @param distribution A \code{character} string. The distribution function of the response.
+#'        Must be "AUTO", "bernoulli", "multinomial", "poisson", "gamma", "tweedie",
+#'        "laplace", "huber" or "gaussian"
+#' @param tweedie_power Tweedie power (only for Tweedie distribution, must be between 1 and 2)
 #' @param score_interval Shortest time interval (in secs) between model scoring
 #' @param score_training_samples Number of training set samples for scoring (0 for all)
 #' @param score_validation_samples Number of validation set samples for scoring (0 for all)
@@ -61,7 +64,7 @@
 #'        disable)
 #' @param quiet_mode Enable quiet mode for less output to standard output
 #' @param max_confusion_matrix_size Max. size (number of classes) for confusion matrices to be shown
-#' @param max_hit_ratio_k Max number (top K) of predictions to use for hit ration computation(for
+#' @param max_hit_ratio_k Max number (top K) of predictions to use for hit ratio computation(for
 #'        multi-class only, 0 to disable)
 #' @param balance_classes Balance training data class counts via over/under-sampling (for imbalanced
 #'        data)
@@ -75,7 +78,7 @@
 #' @param variable_importances Compute variable importances for input features (Gedeon method) - can
 #'        be slow for large networks)
 #' @param fast_mode Enable fast mode (minor approximations in back-propagation)
-#' @param ignore_const_cols Ignore constant columns (no information can be gained anwyay)
+#' @param ignore_const_cols Ignore constant columns (no information can be gained anyway)
 #' @param force_load_balance Force extra load balancing to increase training speed for small
 #'        datasets (to keep all cores busy)
 #' @param replicate_training_data Replicate the entire training dataset onto every node for faster
@@ -85,16 +88,21 @@
 #'        replicated and train_samples_per_iteration is close to \eqn{numRows*numNodes}
 #' @param sparse Sparse data handling (Experimental)
 #' @param col_major Use a column major weight matrix for input layer. Can speed up forward
-#'        proagation, but might slow down backpropagation (Experimental)
+#'        propagation, but might slow down backpropagation (Experimental)
 #' @param average_activation Average activation for sparse auto-encoder (Experimental)
 #' @param sparsity_beta Sparsity regularization (Experimental)
 #' @param max_categorical_features Max. number of categorical features, enforced via hashing
 #'        Experimental)
-#' @param reproducible Force reproducibility on small data (will be slow - only uses 1 thread)
+#' @param reproducible Force reproducibility on small data (requires setting the \code{seed} argument and this will be slow - only uses 1 thread)
 #' @param export_weights_and_biases Whether to export Neural Network weights and biases to H2O
 #'        Frames"
 #' @param offset_column Specify the offset column.
 #' @param weights_column Specify the weights column.
+#' @param nfolds (Optional) Number of folds for cross-validation. If \code{nfolds >= 2}, then \code{validation} must remain empty.
+#' @param fold_column (Optional) Column with cross-validation fold index assignment per observation
+#' @param fold_assignment Cross-validation fold assignment scheme, if fold_column is not specified
+#'        Must be "AUTO", "Random" or "Modulo"
+#' @param keep_cross_validation_predictions Whether to keep the predictions of the cross-validation models
 #' @param ... extra parameters to pass onto functions (not implemented)
 #' @seealso \code{\link{predict.H2OModel}} for prediction.
 #' @examples
@@ -110,7 +118,6 @@
 h2o.deeplearning <- function(x, y, training_frame,
                              model_id = "",
                              overwrite_with_best_model,
-                             n_folds = 0,
                              validation_frame,
                              checkpoint,
                              autoencoder = FALSE,
@@ -138,6 +145,8 @@ h2o.deeplearning <- function(x, y, training_frame,
                              initial_weight_distribution = c("UniformAdaptive", "Uniform", "Normal"),
                              initial_weight_scale = 1,
                              loss = c("Automatic", "CrossEntropy", "MeanSquare", "Absolute", "Huber"),
+                             distribution = c("AUTO","gaussian", "bernoulli", "multinomial", "poisson", "gamma", "tweedie", "laplace", "huber"),
+                             tweedie_power = 1.5,
                              score_interval = 5,
                              score_training_samples,
                              score_validation_samples,
@@ -168,6 +177,10 @@ h2o.deeplearning <- function(x, y, training_frame,
                              export_weights_and_biases=FALSE,
                              offset_column = NULL,
                              weights_column = NULL,
+                             nfolds = 0,
+                             fold_column = NULL,
+                             fold_assignment = c("AUTO","Random","Modulo"),
+                             keep_cross_validation_predictions = FALSE,
                              ...)
 {
   # Pass over ellipse parameters and deprecated parameters
@@ -190,16 +203,17 @@ h2o.deeplearning <- function(x, y, training_frame,
   parms <- list()
   parms$training_frame <- training_frame
   args <- .verify_dataxy(training_frame, x, y, autoencoder)
-  if( !missing(offset_column) )  args$x_ignore <- args$x_ignore[!( offset_column == args$x_ignore )]
-  if( !missing(weights_column) ) args$x_ignore <- args$x_ignore[!( weights_column == args$x_ignore )]
+  if( !missing(offset_column) && !is.null(offset_column))  args$x_ignore <- args$x_ignore[!( offset_column == args$x_ignore )]
+  if( !missing(weights_column) && !is.null(weights_column)) args$x_ignore <- args$x_ignore[!( weights_column == args$x_ignore )]
+  if( !missing(fold_column) && !is.null(fold_column)) args$x_ignore <- args$x_ignore[!( fold_column == args$x_ignore )]
   parms$response_column <- args$y
   parms$ignored_columns <- args$x_ignore
   if(!missing(model_id))
     parms$model_id <- model_id
   if(!missing(overwrite_with_best_model))
     parms$overwrite_with_best_model <- overwrite_with_best_model
-  if(!missing(n_folds))
-    parms$n_folds <- n_folds
+  if(!missing(nfolds))
+    parms$nfolds <- nfolds
   if(!missing(validation_frame))
     parms$validation_frame <- validation_frame
   if(!missing(checkpoint))
@@ -254,6 +268,10 @@ h2o.deeplearning <- function(x, y, training_frame,
     parms$initial_weight_scale <- initial_weight_scale
   if(!missing(loss))
     parms$loss <- loss
+  if (!missing(distribution))
+    parms$distribution <- distribution
+  if (!missing(tweedie_power))
+    parms$tweedie_power <- tweedie_power
   if(!missing(score_interval))
     parms$score_interval <- score_interval
   if(!missing(score_training_samples))
@@ -312,13 +330,15 @@ h2o.deeplearning <- function(x, y, training_frame,
     parms$export_weights_and_biases <- export_weights_and_biases
   if( !missing(offset_column) )             parms$offset_column          <- offset_column
   if( !missing(weights_column) )            parms$weights_column         <- weights_column
-
+  if( !missing(fold_column) )               parms$fold_column            <- fold_column
+  if( !missing(fold_assignment) )           parms$fold_assignment        <- fold_assignment
+  if( !missing(keep_cross_validation_predictions) )  parms$keep_cross_validation_predictions  <- keep_cross_validation_predictions
   .h2o.createModel(training_frame@conn, 'deeplearning', parms)
 }
 
 #' Anomaly Detection via H2O Deep Learning Model
 #'
-#' Detect anomalies in a H2O dataset using a H2O deep learning model with
+#' Detect anomalies in an H2O dataset using an H2O deep learning model with
 #' auto-encoding.
 #'
 #' @param object An \linkS4class{H2OAutoEncoderModel} object that represents the
@@ -328,7 +348,7 @@ h2o.deeplearning <- function(x, y, training_frame,
 #'         reconstruction MSE.
 #' @seealso \code{\link{h2o.deeplearning}} for making an H2OAutoEncoderModel.
 #' @examples
-#' \dontrun{
+#' \donttest{
 #' library(h2o)
 #' localH2O = h2o.init()
 #' prosPath = system.file("extdata", "prostate.csv", package = "h2o")
@@ -349,7 +369,7 @@ h2o.anomaly <- function(object, data) {
 
 #' Feature Generation via H2O Deep Learning Model
 #'
-#' Extract the non-linea feature from an H2O data set using an H2O deep learning
+#' Extract the non-linear feature from an H2O data set using an H2O deep learning
 #' model.
 #' @param object An \linkS4class{H2OModel} object that represents the deep
 #' learning model to be used for feature extraction.
@@ -359,7 +379,7 @@ h2o.anomaly <- function(object, data) {
 #'         number of units in the hidden layer of the specified index.
 #' @seealso \code{link{h2o.deeplearning}} for making deep learning models.
 #' @examples
-#' \dontrun{
+#' \donttest{
 #' library(h2o)
 #' localH2O = h2o.init()
 #' prosPath = system.file("extdata", "prostate.csv", package = "h2o")
